/* Author: Clauirton Siebra <c.siebra@ed.ac.uk>
 * Updated: Mon Feb 23 12:45:50 2003 by Clauirton Siebra
 * Copyright: (c) 2003, AIAI, University of Edinburgh
 */

package ix.ip2.map;

import java.awt.Color;
import java.util.*;
import java.lang.reflect.Array;

import com.bbn.openmap.Layer;
import com.bbn.openmap.layer.OMGraphicHandlerLayer;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.event.*;

import ix.icore.process.event.*; 
import ix.icore.IXAgent;
import ix.util.lisp.*;
import ix.ip2.*;

public class IxRouterLayer extends OMGraphicHandlerLayer implements ProcessStatusListener {

    private Color routeColor = Color.black;
    private OMGraphicList omgraphics; 
    private Projection proj;
    private Hashtable routes;
    private Ip2 agent;
    private int lineType = OMGraphic.LINETYPE_STRAIGHT; // OMGraphic.LINETYPE_STRAIGHT

    public IxRouterLayer() {
        super();       
	omgraphics = (OMGraphicList)getList();
	routes = new Hashtable();
	agent = (Ip2) IXAgent.getAgent();
	(agent.getModelManager()).addProcessStatusListener(this);
    }

    public void setProperties(String prefix, Properties props) {
        super.setProperties(prefix, props);
	//routeColor = Color.getColor(props.getProperty(prefix + ".routeColor", Color.black));
	//lineType = props.getProperty(prefix + ".lineType",OMGraphic.LINETYPE_STRAIGHT);
    }

    public void paint(java.awt.Graphics g) {
        omgraphics.render(g);
    }

    public void projectionChanged(ProjectionEvent pe) {
        Projection oldProj = proj;
        proj = setProjection(pe);
        if (proj != null) {
            getList().generate(proj);
            repaint();
        }
        else
            proj = oldProj;
    
        fireStatusUpdate(LayerStatusEvent.FINISH_WORKING);
    }   

    public OMGraphicList getList() {
        OMGraphicList list = super.getList();
        if (list == null) {
            list = new OMGraphicList();
            super.setList(list);
        }
        return list;
    }

    public void reset(){
        for(int i=omgraphics.size()-1;i>=0;i--){
            omgraphics.removeOMGraphicAt(i);
        }
        getList().generate(proj);
        repaint();
        routes.clear();
    }

    // ProcessStatusListener    

    public void stateChange(ProcessStatusEvent event, Map delta) {
	try{
	    for (Iterator i = delta.entrySet().iterator(); i.hasNext();) {
		Map.Entry e = (Map.Entry)i.next();
		LList pattern = (LList)e.getKey(); 

		LList property = ((LList)pattern.clone()).delete(pattern.get(1));

		if( ((property.get(0)).toString()).equals("route") ) {
		    OMPoly path = null;
		    LList value = (LList) e.getValue();
		    float pointsF[] = new float[value.size()*2];

		    for(int x=0;x<value.size();x++){
			LList point = (LList) value.get(x);
			pointsF[2*x] = Float.parseFloat((point.get(0)).toString());
			pointsF[2*x+1] = Float.parseFloat((point.get(1)).toString());
		    }

		    if (routes.containsKey(pattern.get(1))) {
			path = (OMPoly) routes.get(pattern.get(1)); 
			path.setLocation(pointsF,OMGraphic.DECIMAL_DEGREES);
		    }
		    else {
			path = new OMPoly(pointsF,OMGraphic.DECIMAL_DEGREES,lineType);
			path.setLinePaint(routeColor);	
			omgraphics.addOMGraphic(path);
			routes.put(pattern.get(1),path);
		    }
		}
		getList().generate(proj);
		repaint();
	    }
	}catch(Exception exception){System.err.println("Syntax error in route");}
    }

    public void stateDeletion(ProcessStatusEvent event, java.util.Map delta){
	for (Iterator i = delta.entrySet().iterator(); i.hasNext();) {
	    Map.Entry e = (Map.Entry)i.next();
	    LList pattern = (LList)e.getKey(); 
	    LList property = ((LList)pattern.clone()).delete(pattern.get(1));

	    if( ((property.get(0)).toString()).equals("route") ) {
	        if (routes.containsKey(pattern.get(1))) {
		    omgraphics.remove((OMPoly)routes.get(pattern.get(1)));
		    routes.remove(pattern.get(1));
                    getList().generate(proj);
                    repaint();
		}
	    }
        }
    }

    public void newBindings(ProcessStatusEvent e, java.util.Map bindings){}
    public void statusUpdate(ProcessStatusEvent e){}
  
}